/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 * /
 *******************************************************************************/

package org.eclipse.jet.internal.launch;


import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.ILaunchConfigurationDelegate;
import org.eclipse.jet.internal.runtime.RuntimeLoggerContextExtender;
import org.eclipse.jet.transform.JETLaunchConstants;


/**
 * Create an Eclipse Launch delegate for JET2 Transformations
 *
 */
public class JET2TransformationDelegate implements ILaunchConfigurationDelegate
{

  /**
   * 
   */
  public JET2TransformationDelegate()
  {
    super();
  }

  /**
   * @see org.eclipse.debug.core.model.ILaunchConfigurationDelegate#launch(org.eclipse.debug.core.ILaunchConfiguration, java.lang.String, org.eclipse.debug.core.ILaunch, org.eclipse.core.runtime.IProgressMonitor)
   */
  public void launch(ILaunchConfiguration configuration, String mode, ILaunch launch, IProgressMonitor monitor) throws CoreException
  {

    String id = getId(configuration);
    IResource source = getSource(configuration);
    int logFilterLevel = getLogFilterLevel(configuration);
    final JETProcess process = new JETProcess(launch, id, source, logFilterLevel, monitor);
    launch.addProcess(process);
    
    process.run();
  }

  private IResource getSource(ILaunchConfiguration configuration) throws CoreException
  {
    String sourcePath = configuration.getAttribute(JETLaunchConstants.SOURCE, (String)null);
    IWorkspaceRoot wsRoot = ResourcesPlugin.getWorkspace().getRoot();
    IResource resource = wsRoot.findMember(sourcePath);
    if (resource == null)
    {
      throw new IllegalArgumentException();
    }
    return resource;
  }

  private String getId(ILaunchConfiguration configuration) throws CoreException
  {
    String id = configuration.getAttribute(JETLaunchConstants.ID, (String)null);
    if (id == null)
    {
      throw new IllegalArgumentException();
    }
    return id;
  }

  private int getLogFilterLevel(ILaunchConfiguration configuration) throws CoreException
  {
    return configuration.getAttribute(JETLaunchConstants.LOG_FILTER_LEVEL, RuntimeLoggerContextExtender.INFO_LEVEL);
  }
}
